@@ -27,47 +27,23 @@ test_that("brk_manual", {
 test_that("brk_left, brk_right", {
   expect_identical(
           brk_res(brk_left(1:3)),
-          santoku:::create_breaks(1:3, c(TRUE, TRUE, FALSE))
-        )
-  expect_identical(
-          brk_res(brk_left(1:3, close_end = FALSE)),
           santoku:::create_breaks(1:3, c(TRUE, TRUE, TRUE))
         )
   expect_identical(
-          brk_res(brk_right(1:3)),
-          santoku:::create_breaks(1:3, c(TRUE, FALSE, FALSE))
+          brk_res(brk_left(1:3), close_end = TRUE),
+          santoku:::create_breaks(1:3, c(TRUE, TRUE, FALSE))
         )
   expect_identical(
-          brk_res(brk_right(1:3, close_end = FALSE)),
+          brk_res(brk_right(1:3)),
           santoku:::create_breaks(1:3, c(FALSE, FALSE, FALSE))
         )
+  expect_identical(
+          brk_res(brk_right(1:3), close_end = TRUE),
+          santoku:::create_breaks(1:3, c(TRUE, FALSE, FALSE))
+        )
 
   expect_false(
-    anyNA(chop(1:5, brk_left(1:5, FALSE)))
-  )
-})
-
-
-test_that("brk_left/right wrappers", {
-  expect_identical(
-    brk_res(brk_right(brk_manual(1:3, left = rep(TRUE, 3)))),
-    brk_res(brk_right(1:3))
-  )
-  expect_identical(
-    brk_res(brk_left(brk_manual(1:3, left = rep(FALSE, 3)))),
-    brk_res(brk_left(1:3))
-  )
-  expect_identical(
-    brk_res(brk_right(brk_manual(1:3, left = rep(TRUE, 3)), FALSE)),
-    brk_res(brk_right(1:3, FALSE))
-  )
-  expect_identical(
-    brk_res(brk_left(brk_manual(1:3, left = rep(FALSE, 3)), FALSE)),
-      brk_res(brk_left(1:3, FALSE))
-  )
-  expect_identical(
-    brk_res(brk_left(brk_quantiles(1:3/3))),
-    brk_res(brk_quantiles(1:3/3))
+    anyNA(chop(1:5, brk_left(1:5)))
   )
 })
 
@@ -82,52 +58,49 @@ test_that("brk_n", {
 })
 
 
+test_that("bugfix: brk_n shouldn't error with too many non-unique values", {
+  expect_error(brk_res(brk_n(2), c(1, 1, 1, 1, 5, 5, 5, 5)), regexp = NA)
+})
+
+
 test_that("brk_width", {
-  b <- brk_width(1)(0.5:1.5, FALSE)
+  b <- brk_res(brk_width(1), 0.5:1.5)
   expect_equal(diff(as.vector(b)), 1)
 
   width <- runif(1)
-  b <- brk_width(width)(0.5:1.5, FALSE)
+  b <- brk_res(brk_width(width), 0.5:1.5)
   bvec <- as.vector(b)
   expect_equal(diff(bvec)[1], width)
   expect_equal(bvec[1], 0.5)
 
-  b <- brk_width(1)(rep(NA, 2), FALSE)
+  b <- brk_res(brk_width(1), rep(NA, 2))
   expect_identical(as.vector(b), c(-Inf, Inf))
 
-  b <- brk_width(1)(c(Inf, -Inf, NA), FALSE)
+  b <- brk_res(brk_width(1), c(Inf, -Inf, NA))
   expect_identical(as.vector(b), c(-Inf, Inf))
 
-  b <- brk_width(1)(c(NA, 2, 4, NA), FALSE)
+  b <- brk_res(brk_width(1), c(NA, 2, 4, NA))
   expect_equal(diff(as.vector(b))[1], 1)
 })
 
 
 test_that("brk_evenly", {
-  expect_identical(
-          brk_evenly(5)(0:10, FALSE),
-          brk_width(2)(0:10, FALSE)
-        )
+  b <- brk_res(brk_evenly(5), 0:10)
+  expect_identical(as.vector(b), c(0, 2, 4, 6, 8, 10))
 })
 
 
 test_that("brk_mean_sd", {
   x <- rnorm(100)
-  expect_silent(b <- brk_mean_sd(3)(x, FALSE))
+  expect_silent(b <- brk_res(brk_mean_sd(3), x = x))
   m <- mean(x)
   sd <- sd(x)
   sd_ints <- seq(m - 3 * sd, m + 3 * sd, sd)
   expect_equal(as.numeric(b), sd_ints)
 
-  b <- brk_mean_sd(1)(x, NULL)
-  expect_equal(
-          attr(b, "break_labels"),
-          c("-Inf", "-1 sd", "0 sd", "1 sd", "Inf")
-        )
-
-  expect_silent(brk_mean_sd(3)(rep(NA, 2), FALSE))
-  expect_silent(brk_mean_sd(3)(rep(1, 3), FALSE))
-  expect_silent(brk_mean_sd(3)(1, FALSE))
+  expect_silent(brk_res(brk_mean_sd(3), x = rep(NA, 2)))
+  expect_silent(brk_res(brk_mean_sd(3), x = rep(1, 3)))
+  expect_silent(brk_res(brk_mean_sd(3), x = 1))
 })
 
 
@@ -135,18 +108,27 @@ test_that("brk_quantiles", {
   expect_silent(brk_res(brk_quantiles(1:3/4)))
 
   x <- 1:10
-  brks <- brk_quantiles(1:3/4)(x, FALSE)
+  brks <- brk_quantiles(1:3/4)(x, FALSE, TRUE, FALSE)
   expect_equivalent(c(brks), quantile(x, 1:3/4))
 
-  expect_silent(brks <- brk_quantiles(numeric(0))(x, TRUE))
+  expect_silent(brks <- brk_quantiles(numeric(0))(x, TRUE, TRUE, FALSE))
   expect_equivalent(c(brks), c(-Inf, Inf))
 
   x <- rep(1, 5)
-  brks <- brk_quantiles(1:3/4)(x, FALSE)
+  brks <- brk_quantiles(1:3/4)(x, FALSE, TRUE, FALSE)
   expect_equivalent(c(brks), unique(quantile(x, 1:3/4)))
 })
 
 
+test_that("brk_equally", {
+  expect_silent(brk_res(brk_equally(5)))
+  expect_error(brk_equally(4.5))
+
+  brks <- brk_res(brk_equally(3))
+  expect_equivalent(brks, brk_res(brk_quantiles(0:3/3)))
+})
+
+
 test_that("printing", {
   b <- brk_left(1:3)
   expect_output(print(b))
@@ -155,9 +137,3 @@ test_that("printing", {
   expect_output(print(b_empty))
 })
 
-
-test_that("systematic tests", {
-  # input of normal; unsorted; with NA; with +-Inf; only 1 break;
-  # non-numeric; length 0
-
-})
